#!/usr/bin/perl -w
use strict;

use Getopt::Long;
my ($printMatrix, $useSymmetry, $fillCols, $maxp);
GetOptions(
	"print-matrix"	=> \$printMatrix,
	"use-symmetry"	=> \$useSymmetry,
	"fill-cols=i"	=> \$fillCols,
	"max-p=f"		=> \$maxp
)	or exit;

use lib '/home/vagabond/research/software/channels/';
use Channel;

my %data;
my %noobs;
my ($p, $user, $observ);

while(<>) {
	chomp;

	/Model checking:.*noobs.*user(\d+)/ && do {
		$observ = "noobs";
		$user = $1;
		next;
	};

	/Model checking:.*observ(\d+).*user(\d+)/ && do {
		$observ = $1;
		$user = $2;
		next;
	};

	/constants: pf=(\d+(\.\d*)?)/ && do {
		$p = $1/100;
		next;
	};

	/Result: (\d(\.\d*)?)(E-(\d*))?/ && do {
		my $res = $1;
		$res *=  10 ** (-$4)	if defined $4;

		if($observ eq "noobs") {
			$noobs{$p} = [] 				unless exists $noobs{$p};
			$noobs{$p}[$user] = $res;
			next;
		}

		$data{$p} = [] 				unless exists $data{$p};
		$data{$p}[$user] = []		unless exists $data{$p}[$user-1];
		$data{$p}[$user][$observ] = $res;
	};
}

# if only two columns are given, fill the rest
if($fillCols) {
	for my $m (values %data) {
		for my $i (0..@$m-1) {
			my $val = $m->[$i][$i == 0 ? 1 : 0];
			for my $j (1..$fillCols-1) {
				next if $i eq $j;
				$m->[$i][$j] = $val;
			}
		}
	}
}

# normalize rows
# check that the normalization factor is 1-noobs
#
for my $p (keys %data) {
	my $m = $data{$p};
	my $noobs = $noobs{$p};

	for my $i (0..@$m-1) {
		my $sum = Channel::sum($m->[$i]);
		die "error (p=$p) sum ($sum) different than 1-noobs (" . (1-$noobs->[$i]) . ")"
			if abs($sum - 1 + $noobs->[$i]) > 0.0001;
		$_ /= $sum for @{$m->[$i]};
	}
}

use Data::Dumper;

my @sortp = sort { $a <=> $b } keys %data;
my $e = 0.00001;

#print "$_:  ", Dumper($data{$_})
#	for @sortp;
#exit;
#my $matrix = $data{'0.7'};
#my $chan = new Channel $matrix;
#print $chan->to_string;
#exit;

for $p (@sortp) {
	last if defined $maxp && $p > $maxp;

	my $matrix = $data{$p};
	my $chan = new Channel $data{$p};

	#print_matrix($matrix)
	print $chan->to_string
		if $printMatrix;

	# check that all rows add-up to 1
	$chan->test_matrix
		or print "WARNING: row's sum is not 1: " . $chan->to_string;

	my $capacity;
	if($useSymmetry) {
		$capacity = $chan->capacity_sym;
	} else {
		($capacity) = $chan->capacity($e);
	}

	printf "%.6f   %.6f\n", $p, $capacity;
}



